from typing import Literal, cast

ContextTypeResourceType = Literal["context_type"]

CONTEXT_TYPE_RESOURCE_TYPE_VALUES: set[ContextTypeResourceType] = {
    "context_type",
}


def check_context_type_resource_type(value: str) -> ContextTypeResourceType:
    if value in CONTEXT_TYPE_RESOURCE_TYPE_VALUES:
        return cast(ContextTypeResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {CONTEXT_TYPE_RESOURCE_TYPE_VALUES!r}")



from typing import Literal, cast

ProductResourceType = Literal["product"]

PRODUCT_RESOURCE_TYPE_VALUES: set[ProductResourceType] = {
    "product",
}


def check_product_resource_type(value: str) -> ProductResourceType:
    if value in PRODUCT_RESOURCE_TYPE_VALUES:
        return cast(ProductResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {PRODUCT_RESOURCE_TYPE_VALUES!r}")



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, TYPE_CHECKING

from attrs import define as _attrs_define
from attrs import field as _attrs_field


if TYPE_CHECKING:
    from ..models.flag_resource import FlagResource


T = TypeVar("T", bound="FlagResponse")


@_attrs_define
class FlagResponse:
    """
    Attributes:
        data (FlagResource):  Example: {'attributes': {'created_at': '2026-03-27T10:00:00Z', 'default': False,
            'description': 'Enable dark mode for the application UI', 'environments': {'production': {'default': False,
            'enabled': True, 'rules': [{'description': 'Beta users get dark mode', 'logic': {'attribute': 'beta', 'op':
            'eq', 'value': True}, 'value': True}]}}, 'key': 'dark_mode', 'name': 'Dark Mode', 'type': 'BOOLEAN',
            'updated_at': '2026-03-27T10:00:00Z', 'values': [{'name': 'on', 'value': True}, {'name': 'off', 'value':
            False}]}, 'id': '550e8400-e29b-41d4-a716-446655440000', 'type': 'flag'}.
    """

    data: FlagResource
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        data = self.data.to_dict()

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "data": data,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.flag_resource import FlagResource

        d = dict(src_dict)
        data = FlagResource.from_dict(d.pop("data"))

        flag_response = cls(
            data=data,
        )

        flag_response.additional_properties = d
        return flag_response

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, cast

PlanResourceType = Literal["plan"]

PLAN_RESOURCE_TYPE_VALUES: set[PlanResourceType] = {
    "plan",
}


def check_plan_resource_type(value: str) -> PlanResourceType:
    if value in PLAN_RESOURCE_TYPE_VALUES:
        return cast(PlanResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {PLAN_RESOURCE_TYPE_VALUES!r}")



from typing import Literal, cast

EnvironmentResourceType = Literal["environment"]

ENVIRONMENT_RESOURCE_TYPE_VALUES: set[EnvironmentResourceType] = {
    "environment",
}


def check_environment_resource_type(value: str) -> EnvironmentResourceType:
    if value in ENVIRONMENT_RESOURCE_TYPE_VALUES:
        return cast(EnvironmentResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {ENVIRONMENT_RESOURCE_TYPE_VALUES!r}")



from __future__ import annotations

from collections.abc import Mapping
from typing import Any, TypeVar, TYPE_CHECKING

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

from ..models.product_resource_type import check_product_resource_type
from ..models.product_resource_type import ProductResourceType
from typing import cast

if TYPE_CHECKING:
    from ..models.product import Product


T = TypeVar("T", bound="ProductResource")


@_attrs_define
class ProductResource:
    """
    Attributes:
        type_ (ProductResourceType):
        attributes (Product):
        id (None | str | Unset):
    """

    type_: ProductResourceType
    attributes: Product
    id: None | str | Unset = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        type_: str = self.type_

        attributes = self.attributes.to_dict()

        id: None | str | Unset
        if isinstance(self.id, Unset):
            id = UNSET
        else:
            id = self.id

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "type": type_,
                "attributes": attributes,
            }
        )
        if id is not UNSET:
            field_dict["id"] = id

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: Mapping[str, Any]) -> T:
        from ..models.product import Product

        d = dict(src_dict)
        type_ = check_product_resource_type(d.pop("type"))

        attributes = Product.from_dict(d.pop("attributes"))

        def _parse_id(data: object) -> None | str | Unset:
            if data is None:
                return data
            if isinstance(data, Unset):
                return data
            return cast(None | str | Unset, data)

        id = _parse_id(d.pop("id", UNSET))

        product_resource = cls(
            type_=type_,
            attributes=attributes,
            id=id,
        )

        product_resource.additional_properties = d
        return product_resource

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Literal, cast

UserResourceType = Literal["user"]

USER_RESOURCE_TYPE_VALUES: set[UserResourceType] = {
    "user",
}


def check_user_resource_type(value: str) -> UserResourceType:
    if value in USER_RESOURCE_TYPE_VALUES:
        return cast(UserResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {USER_RESOURCE_TYPE_VALUES!r}")



from typing import Literal, cast

ApiKeyResourceType = Literal["api_key"]

API_KEY_RESOURCE_TYPE_VALUES: set[ApiKeyResourceType] = {
    "api_key",
}


def check_api_key_resource_type(value: str) -> ApiKeyResourceType:
    if value in API_KEY_RESOURCE_TYPE_VALUES:
        return cast(ApiKeyResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {API_KEY_RESOURCE_TYPE_VALUES!r}")



from typing import Literal, cast

InvitationResourceType = Literal["invitation"]

INVITATION_RESOURCE_TYPE_VALUES: set[InvitationResourceType] = {
    "invitation",
}


def check_invitation_resource_type(value: str) -> InvitationResourceType:
    if value in INVITATION_RESOURCE_TYPE_VALUES:
        return cast(InvitationResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {INVITATION_RESOURCE_TYPE_VALUES!r}")



from typing import Literal, cast

ServiceResourceType = Literal["service"]

SERVICE_RESOURCE_TYPE_VALUES: set[ServiceResourceType] = {
    "service",
}


def check_service_resource_type(value: str) -> ServiceResourceType:
    if value in SERVICE_RESOURCE_TYPE_VALUES:
        return cast(ServiceResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {SERVICE_RESOURCE_TYPE_VALUES!r}")



from typing import Literal, cast

AccountResourceType = Literal["account"]

ACCOUNT_RESOURCE_TYPE_VALUES: set[AccountResourceType] = {
    "account",
}


def check_account_resource_type(value: str) -> AccountResourceType:
    if value in ACCOUNT_RESOURCE_TYPE_VALUES:
        return cast(AccountResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {ACCOUNT_RESOURCE_TYPE_VALUES!r}")



from typing import Literal, cast

ContextResourceType = Literal["context"]

CONTEXT_RESOURCE_TYPE_VALUES: set[ContextResourceType] = {
    "context",
}


def check_context_resource_type(value: str) -> ContextResourceType:
    if value in CONTEXT_RESOURCE_TYPE_VALUES:
        return cast(ContextResourceType, value)
    raise TypeError(f"Unexpected value {value!r}. Expected one of {CONTEXT_RESOURCE_TYPE_VALUES!r}")



from http import HTTPStatus
from typing import Any

import httpx

from ...client import AuthenticatedClient, Client
from ...types import Response, UNSET
from ... import errors

from ...models.error_response import ErrorResponse
from ...models.http_validation_error import HTTPValidationError
from ...models.logger_list_response import LoggerListResponse
from ...types import Unset


def _get_kwargs(
    *,
    filterkey: None | str | Unset = UNSET,
    filtermanaged: bool | None | Unset = UNSET,
) -> dict[str, Any]:

    params: dict[str, Any] = {}

    json_filterkey: None | str | Unset
    if isinstance(filterkey, Unset):
        json_filterkey = UNSET
    else:
        json_filterkey = filterkey
    params["filter[key]"] = json_filterkey

    json_filtermanaged: bool | None | Unset
    if isinstance(filtermanaged, Unset):
        json_filtermanaged = UNSET
    else:
        json_filtermanaged = filtermanaged
    params["filter[managed]"] = json_filtermanaged

    params = {k: v for k, v in params.items() if v is not UNSET and v is not None}

    _kwargs: dict[str, Any] = {
        "method": "get",
        "url": "/api/v1/loggers",
        "params": params,
    }

    return _kwargs


def _parse_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> ErrorResponse | HTTPValidationError | LoggerListResponse | None:
    if response.status_code == 200:
        response_200 = LoggerListResponse.from_dict(response.json())

        return response_200

    if response.status_code == 400:
        response_400 = ErrorResponse.from_dict(response.json())

        return response_400

    if response.status_code == 401:
        response_401 = ErrorResponse.from_dict(response.json())

        return response_401

    if response.status_code == 404:
        response_404 = ErrorResponse.from_dict(response.json())

        return response_404

    if response.status_code == 422:
        response_422 = HTTPValidationError.from_dict(response.json())

        return response_422

    if response.status_code == 429:
        response_429 = ErrorResponse.from_dict(response.json())

        return response_429

    if client.raise_on_unexpected_status:
        raise errors.UnexpectedStatus(response.status_code, response.content)
    else:
        return None


def _build_response(
    *, client: AuthenticatedClient | Client, response: httpx.Response
) -> Response[ErrorResponse | HTTPValidationError | LoggerListResponse]:
    return Response(
        status_code=HTTPStatus(response.status_code),
        content=response.content,
        headers=response.headers,
        parsed=_parse_response(client=client, response=response),
    )


def sync_detailed(
    *,
    client: AuthenticatedClient,
    filterkey: None | str | Unset = UNSET,
    filtermanaged: bool | None | Unset = UNSET,
) -> Response[ErrorResponse | HTTPValidationError | LoggerListResponse]:
    """List Loggers

    Args:
        filterkey (None | str | Unset):
        filtermanaged (bool | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | HTTPValidationError | LoggerListResponse]
    """

    kwargs = _get_kwargs(
        filterkey=filterkey,
        filtermanaged=filtermanaged,
    )

    response = client.get_httpx_client().request(
        **kwargs,
    )

    return _build_response(client=client, response=response)


def sync(
    *,
    client: AuthenticatedClient,
    filterkey: None | str | Unset = UNSET,
    filtermanaged: bool | None | Unset = UNSET,
) -> ErrorResponse | HTTPValidationError | LoggerListResponse | None:
    """List Loggers

    Args:
        filterkey (None | str | Unset):
        filtermanaged (bool | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | HTTPValidationError | LoggerListResponse
    """

    return sync_detailed(
        client=client,
        filterkey=filterkey,
        filtermanaged=filtermanaged,
    ).parsed


async def asyncio_detailed(
    *,
    client: AuthenticatedClient,
    filterkey: None | str | Unset = UNSET,
    filtermanaged: bool | None | Unset = UNSET,
) -> Response[ErrorResponse | HTTPValidationError | LoggerListResponse]:
    """List Loggers

    Args:
        filterkey (None | str | Unset):
        filtermanaged (bool | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        Response[ErrorResponse | HTTPValidationError | LoggerListResponse]
    """

    kwargs = _get_kwargs(
        filterkey=filterkey,
        filtermanaged=filtermanaged,
    )

    response = await client.get_async_httpx_client().request(**kwargs)

    return _build_response(client=client, response=response)


async def asyncio(
    *,
    client: AuthenticatedClient,
    filterkey: None | str | Unset = UNSET,
    filtermanaged: bool | None | Unset = UNSET,
) -> ErrorResponse | HTTPValidationError | LoggerListResponse | None:
    """List Loggers

    Args:
        filterkey (None | str | Unset):
        filtermanaged (bool | None | Unset):

    Raises:
        errors.UnexpectedStatus: If the server returns an undocumented status code and Client.raise_on_unexpected_status is True.
        httpx.TimeoutException: If the request takes longer than Client.timeout.

    Returns:
        ErrorResponse | HTTPValidationError | LoggerListResponse
    """

    return (
        await asyncio_detailed(
            client=client,
            filterkey=filterkey,
            filtermanaged=filtermanaged,
        )
    ).parsed
